@@ -30,6 +30,8 @@
 #include <unistd.h>
 #include <time.h>
 
+#define WITH_FQDN6
+
 /* Some function prototypes so that the overly anal GCC doesn't
  * generate warnings */
 js_string *csv2_copy_js(js_string *s);
@@ -1118,6 +1120,77 @@ js_string *csv2_make_ptr_query(js_string *binary_ip) {
         return ptr_label;
 }
 
+/* Given a binary js_string object with an IP in it (e.g. "127.0.0.1"
+   is, in hexadecimal "7f000001" in the input), create the corresponding
+   ptr label as a Blabel ("\x011\x010\x010\x03127\x07in-addr\x04arpa\x00"
+   in the "7f000001" case). */
+
+#ifdef WITH_FQDN6
+js_string *csv2_make_ptr6_query(js_string *binary_ip) {
+        js_string *ptr_label;
+        int len = 0;
+        int counter = 0;
+
+        /* Sanity check */
+        if(js_length(binary_ip) != 16)
+                return 0;
+
+        ptr_label = js_create(256,1); /* Maximum possible length for this */
+        if(ptr_label == 0) {
+                return 0;
+        }
+
+        for(counter = 15; counter >= 0; counter--) {
+                char a;
+                int b;
+
+                a = (*(binary_ip->string + counter));
+
+                /* low nibble */
+                *(ptr_label->string + len) = 1;
+                len++;
+                b = a & 0x0f;
+                if ( b > 9 ) b += 'a' - 10;
+                else         b += '0';
+                *(ptr_label->string + len) = b;
+                len++;
+
+                /* high nibble */
+                *(ptr_label->string + len) = 1;
+                len++;
+                b = (a >> 4) & 0x0f;
+                if ( b > 9 ) b += 'a' - 10;
+                else         b += '0';
+                *(ptr_label->string + len) = b;
+                len++;
+        }
+
+        if(len <= 250) {
+                *(ptr_label->string + len)   =  3;  /* "ipv6" length */
+                *(ptr_label->string + len+1) = 'i'; /* "ipv6 */
+                *(ptr_label->string + len+2) = 'p';
+                *(ptr_label->string + len+3) = '6';
+                len += 4;
+        }
+
+        if(len <= 250) {
+                *(ptr_label->string + len) = 4;     /* "arpa" length */
+                *(ptr_label->string + len+1) = 'a'; /* "ipv6 */
+                *(ptr_label->string + len+2) = 'r';
+                *(ptr_label->string + len+3) = 'p';
+                *(ptr_label->string + len+4) = 'a';
+                len += 5;
+        }
+        if(len <= 255) {
+                *(ptr_label->string + len) = 0; /* null to end dlabel */
+                len++;
+        }
+        ptr_label->unit_size=1;
+        ptr_label->unit_count=len;
+        return ptr_label;
+}
+#endif
+
 /* Function to add an rr in general */
 /* Note: "query" is a Blabel */
 int csv2_add_rr_w(csv2_add_state *state, js_string *query,
@@ -1144,7 +1217,7 @@ int csv2_add_rr_w(csv2_add_state *state, js_string *query,
                 return JS_ERROR;
         }
         /* A-Z, nicely enough, never happen in a domain length label,
-           so we can very quickly make this label lower-case */
+          so we can very quickly make this label lower-case */
         while(counter < cquery->unit_count) {
                 if(*(cquery->string + counter) >= 'A' &&
                    *(cquery->string + counter) <= 'Z') {
@@ -1170,6 +1243,22 @@ int csv2_add_rr_w(csv2_add_state *state, js_string *query,
             js_destroy(cquery);
             return JS_ERROR;
             }
+#ifdef WITH_FQDN6
+        if(rtype == 65766 && state->add_method != 3) {
+                js_string *pq;
+                rtype = RR_AAAA;
+                pq = csv2_make_ptr6_query(data);
+                /* This can not infinitely recurse because we change the
+                * rtype from the magic 65766 rtype */
+                csv2_add_rr_w(state,pq,RR_PTR,ttl,cquery);
+                js_destroy(pq);
+        } else if(rtype == 65766 && state->add_method == 3) {
+                printf("Warning: FQDN6 records are not permitted "
+                       "in the\ndefault zonefile.  This record is ignored.\n");
+                js_destroy(cquery);
+                return JS_ERROR;
+        }
+#endif
         ret = JS_ERROR; /* This should be set to another value */
         switch(state->add_method) {
                 case 1: /* Adding records to MaraDNS-1 style "bighash" */
