@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2007 Sam Trenholme
+/* Copyright (c) 2002-2008 Sam Trenholme
  *
  * TERMS
  *
@@ -39,6 +39,7 @@ extern int max_total;
 extern rr *seenlist[256];
 extern int seenlist_where;
 extern ipv4pair long_packet[512];
+extern int calc_ra_value(); /* Make -Wall happy */
 
 /* If we successfully found a record, spit out that record on the
    udp packet.
@@ -472,7 +473,7 @@ int udpsuccess(rr *where, int id, int sock, struct sockaddr_in *client,
     header.opcode = 0;
     header.tc = 0;
     header.rd = rd_val; /* RDBUG udpsuccess */
-    header.ra = ra_value;
+    header.ra = calc_ra_value(ra_value);
     header.z = 0;
     header.rcode = 0; /* No error */
     header.qdcount = 1;
