@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2007 Sam Trenholme and Albert Lee
+/* Copyright (c) 2002-2008 Sam Trenholme and Albert Lee
  *
  * TERMS
  *
@@ -76,6 +76,8 @@ int dns_records_served = 0;
    to */
 int udp_forward_server = 0;
 
+int dns_port = 53; /* The default port for the zoneserver to listen on */
+
 int no_cname_warnings = 1; /* So we can link to MaraBigHash.o */
 
 /* Signal handler for handling the exit of a child */
@@ -247,7 +249,7 @@ uint16 srng() {
     return 12;
     }
 
-/* Bind to TCP port 53.
+/* Bind to TCP dns_port.
    Input: pointer to socket to bind on, js_string with the dotted-decimal
           ip address to bind to
    Output: JS_ERROR on error, JS_SUCCESS on success
@@ -256,6 +258,7 @@ uint16 srng() {
 int tcpbind(int *sock, uint32 ip) {
     int len_inet; /* Length */
     struct sockaddr_in dns_tcp;
+    int on = 1;
 
     /* Sanity checks */
     if(sock == 0)
@@ -270,11 +273,17 @@ int tcpbind(int *sock, uint32 ip) {
     if((*sock = socket(PF_INET,SOCK_STREAM,0)) == -1) {
         return JS_ERROR;
         }
+    /* Allow the socket to be reused more quickly (makes testing
+     * easier) */
+    if(setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, (char *)&on,
+                  sizeof(on)) == -1) {
+        return JS_ERROR;
+        }
 
     /* Choose an IP and port to bind to */
     memset(&dns_tcp,0,sizeof(dns_tcp));
     dns_tcp.sin_family = AF_INET;
-    dns_tcp.sin_port = htons(53);
+    dns_tcp.sin_port = htons(dns_port);
     if((dns_tcp.sin_addr.s_addr = ip) == INADDR_NONE)
         return JS_ERROR;
 
@@ -292,7 +301,7 @@ int tcpbind(int *sock, uint32 ip) {
     if(listen(*sock,250) == -1)
         return JS_ERROR;
 
-    /* We are now on TCP port 53.  Leave */
+    /* We are now on TCP dns_port.  Leave */
     return JS_SUCCESS;
     }
 
@@ -1014,8 +1023,8 @@ int main(int argc, char **argv) {
     js_string *synth_soa_origin;
 
     /* Kill children processes when we are signaled */
-    if(setpgrp()) {
-        printf(strerror(errno)); /* harderror() would kill the group which may not be correct yet */
+    if(setpgid(0,0)) {
+        printf("%s",strerror(errno)); /* harderror() would kill the group which may not be correct yet */
         return 3;
     }
     signal(SIGTERM,handle_term);
@@ -1159,10 +1168,18 @@ on %s\nFor usage information, type in 'man maradns'" */
 
         mlog(L_CHROOT_SUCCESS); /* "Root directory changed" */
 
-        /* Bind to port 53
+        /* Bind to dns_port
            To Do: use capset to give us privledged bind abilities without
                   needing to be root.
         */
+
+        /* Set the dns_port */
+        dns_port = read_numeric_kvar("dns_port",53);
+        if(dns_port < 1 || dns_port > 65530) {
+            harderror("dns_port must be between 1 and 65530");
+            exit(1);
+        }
+
         if(inetd != 1) { /* If we are a standalone server */
             ipv4pair *bind_addresses;
             int bind_address_iterate;
@@ -1207,7 +1224,7 @@ on %s\nFor usage information, type in 'man maradns'" */
                             dup2(stream1[1],2); /* Stderr redirection */
                             if(tcpbind(&sock,
                         bind_addresses[bind_address_iterate].ip) == JS_ERROR)
-                            harderror(L_BIND); /* "Problem binding to port 53.\nMost likely, another process is already listening on port 53" */
+                            harderror(L_BIND); /* "Problem binding to dns_port.\nMost likely, another process is already listening on dns_port" */
                             break;
                     }
                     bind_address_iterate++;
@@ -1243,7 +1260,8 @@ on %s\nFor usage information, type in 'man maradns'" */
                                     if(FD_ISSET(counter,&read_set)) {
                                         readed = read(counter,buf,1024);
                                         if(readed > 0) {
-                                            write(1,buf,readed);
+                                            int nowarn;
+                                            nowarn = write(1,buf,readed);
                                         } else {
                                             /* No longer exists  */
                                             close(counter);
@@ -1260,7 +1278,7 @@ on %s\nFor usage information, type in 'man maradns'" */
             }
             if(libtcp_create_bind_addrs() == JS_ERROR)
                 harderror("libtcp_create_synthip_addrs");
-            mlog(L_SOCKET_SUCCESS);  /* "Socket opened on TCP port 53" */
+            mlog(L_SOCKET_SUCCESS);  /* "Socket opened on TCP dns_port" */
             }
 
         /* Drop the elevated privileges */
@@ -1417,7 +1435,7 @@ on %s\nFor usage information, type in 'man maradns'" */
           7: Both zone transfer and forward with recursion enabled
         */
         if(verbose >= 2)
-            mlog(L_WAITING); /* "Awaiting data on port 53" */
+            mlog(L_WAITING); /* "Awaiting data on dns_port" */
         connection = gettcp(&sock,zonetransfer_acl,tcpconvert_acl,
                      recursive_acl,500,&permissions);
         if(connection == JS_ERROR)
