@@ -30,9 +30,13 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#ifndef MINGW32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#else
+#include <winsock2.h>
+#endif
 #include "../MaraDns.h"
 /* All of the labels */
 #include "askmara_labels_en.h"
@@ -49,6 +53,8 @@ int verbose_mode = 0; /* Whether to have short or verbose output;
                          1: verbose output; any other value: short output */
 int timeout = 21;
 
+unsigned short dns_port = DNS_PORT;
+
 /* So -Wall doesn't gag */
 extern int decomp_init();
 
@@ -152,7 +158,11 @@ int harderror(char *msg) {
 
 int main(int argc, char **argv) {
     char *server_address = NULL;
+#if defined AUTHONLY && ! defined __CYGWIN__
+    struct sockaddr_in6 server;
+#else
     struct sockaddr_in server;
+#endif
     int len_inet; /* Length */
     int s; /* Socket */
     js_string *outdata; /* Outgoing data */
@@ -168,33 +178,53 @@ int main(int argc, char **argv) {
     int length;
 
     /* Determine what the query string is */
-    if(argc < 2 || argc > 6)
+    verbose_mode = 0;
+    timeout = 31;
+    server_address = "127.0.0.3";
+    argc--;
+    argv++;
+    while(argc > 0) {
+        if(argv[0] [0]== '-' && argv[0][1] == 'v')
+            verbose_mode = 1;
+        else if(argv[0][0] == '-' && argv[0][1] == 'n')
+            nrd = 1;
+        else if(argv[0][0] == '-' && argv[0][1] == 'p') {
+            if(argc < 2) {
+                harderror(L_USAGE);
+                }
+            argc--;
+            argv++;
+            dns_port = atoi(*argv);
+            if(dns_port < 1 || dns_port > 32000) {
+                    harderror("Wrong port number\n");
+                }
+            }
+        else if(argv[0][0] == '-' && argv[0][1] == 't') {
+            if(argc < 2) {
+                harderror(L_USAGE);
+                }
+            argc--;
+            argv++;
+            timeout = atoi(*argv);
+            if(timeout < 1)
+               harderror( L_INVALID_TIMEOUT);
+            verbose_mode = 2;
+            }
+        else if (argv[0][0] != '-')
+            break;
+        else
+            harderror(L_USAGE);
+        argc--;
+        argv++;
+        }
+
+    /* we expect now the query string and possibly the server */
+    if(argc<1)
         harderror(L_USAGE);
 
-    if(argc >= 3 && *argv[1] == '-' && *(argv[1] + 1) == 'n')
-        nrd = 1;
-
-    /* See if we are in verbose mode or not */
-    verbose_mode = 0;
-    if(argc >= 3 + nrd && *argv[1 + nrd] == '-' && *(argv[1 + nrd] + 1) == 'v')
-        verbose_mode = 1;
-
-    /* See if we should allow a user defined timeout or not */
-    timeout = 31;
-    if(argc >= 4 + nrd && *argv[1 + nrd] == '-' && *(argv[1 + nrd] + 1) ==
-       't') {
-         timeout = atoi(argv[2 + nrd]);
-         verbose_mode = 2;
-         }
-
-    if(timeout < 1)
-        harderror(L_INVALID_TIMEOUT);
-
     /* Determine what IP address to connect to */
-    if(argc == 3 + verbose_mode + nrd)
-        server_address = argv[2 + verbose_mode + nrd];
-    else
-        server_address = "127.0.0.3";
+    if(argc>1)
+        server_address = argv[1];
 
     if((indata = js_create(512,1)) == 0)
        harderror(L_JS_CREATE_INDATA);
@@ -204,24 +234,49 @@ int main(int argc, char **argv) {
        harderror(L_JS_CREATE_OUTDATA);
     if((question.qname = js_create(512,1)) == 0)
        harderror(L_JS_CREATE_QNAME);
-    /* Create a TCP socket */
-    if((s = socket(PF_INET,SOCK_STREAM,0)) == -1)
-        harderror(L_SOCKET);
 
     /* Create a socket address */
     memset(&server,0,sizeof(server));
+#if defined AUTHONLY && ! defined __CYGWIN__
+    if ( strchr(server_address,':') != NULL) {
+        /* assume that we have an IPv6 address */
+
+        /* Create a TCP socket */
+        if((s = socket(PF_INET6,SOCK_STREAM,0)) == -1)
+            harderror(L_SOCKET);
+
+        server.sin6_family = AF_INET6;
+        server.sin6_port = htons(dns_port);
+        if( inet_pton(AF_INET6, server_address, &server.sin6_addr) < 1) {
+            harderror(L_MAL_IP);
+            }
+        len_inet = sizeof(struct sockaddr_in6);
+        }
+    else {
+        if((s = socket(PF_INET,SOCK_STREAM,0)) == -1)
+            harderror(L_SOCKET);
+        ((struct sockaddr_in*)&server)->sin_family = AF_INET;
+        ((struct sockaddr_in*)&server)->sin_port = htons(dns_port);
+        if((((struct sockaddr_in*)&server)->sin_addr.s_addr = inet_addr(server_address)) == INADDR_NONE)
+            harderror(L_MAL_IP);
+        len_inet = sizeof(struct sockaddr_in);
+        }
+#else
+    if((s = socket(PF_INET,SOCK_STREAM,0)) == -1)
+        harderror(L_SOCKET);
+
     server.sin_family = AF_INET;
     /* When debugging MaraDNS on systems where I do not have root, it is
        useful to be able to contact a MaraDNS server running on a high
        port number.  However, DNS_PORT should be 53 otherwise (defined
        in MaraDNS.h) */
-    server.sin_port = htons(DNS_PORT);
+    server.sin_port = htons(dns_port);
     if((server.sin_addr.s_addr = inet_addr(server_address)) == INADDR_NONE)
         harderror(L_MAL_IP);
+    len_inet = sizeof(struct sockaddr_in);
+#endif
 
-    len_inet = sizeof(server);
-
-    id = gen_id(argv[1 + verbose_mode]);
+    id = gen_id(argv[0]);
     /* Format a DNS request */
     /* DNS header */
     header.id = id;
@@ -241,15 +296,17 @@ int main(int argc, char **argv) {
     /* Create a DNS question , and put it in raw UDP format */
     /* DNS question -> looking up an A record... */
     question.qclass = 1; /* ...on the internet */
-    if(js_qstr2js(question.qname,argv[1 + verbose_mode + nrd]) == JS_ERROR)
+    if(js_qstr2js(question.qname,argv[0]) == JS_ERROR)
         harderror(L_INVALID_Q); /* Invalid query */
 
     /* Make sure that the csv1-compatible output hashes all non-RR
        output */
     if(verbose_mode != 1)
         printf("# ");
-
-    printf("%s%s%s",L_QUERYING,server_address,L_NEWLINE); /* Querying the server with the IP... */
+    if(dns_port==DNS_PORT)
+        printf("%s%s%s",L_QUERYING,server_address,L_NEWLINE); /* Querying the server with the IP... */
+    else
+        printf("%s%s#%d%s",L_QUERYING,server_address, dns_port,L_NEWLINE); /* Querying the server with the IP... */
 
     /* Make 'Aexample.com.' raw UDP data */
 
@@ -300,8 +357,6 @@ int main(int argc, char **argv) {
            id = header.id + 1;
            }
 
-        printf("DEBUG ID: %d header.id: %d\n",id,header.id);
-
         attempts--;
 
         /* If the id doesn't match, get a reply again */
@@ -473,6 +528,23 @@ int out_answer(js_string *uindata,int *place) {
                    L_DOT,*(uindata->string + *place + 2),
                    L_DOT,*(uindata->string + *place + 3),L_NEWLINE,L_NEWLINE);
             }
+        else if(rr_hdr.type == RR_AAAA) {
+            unsigned short *p;
+            if(uindata->unit_count < *place + 16)
+                return JS_ERROR;
+            /* Display the IP of the data */
+            /* Display the IP of the data */
+            p = (unsigned short*)(uindata->string + *place);
+            printf("%s%d%x:%x:%x:%x:%x:%x:%x:%x%s%s",L_IP,*(uindata->string + *place),
+                       htons(*(p + 0 )),
+                       htons(*(p + 1)),
+                       htons(*(p + 2)),
+                       htons(*(p + 3)),
+                       htons(*(p + 4)),
+                       htons(*(p + 5)),
+                       htons(*(p + 6)),
+                       htons(*(p + 7)),L_NEWLINE,L_NEWLINE);
+            }
         else if(rr_hdr.type == RR_TXT) {
             printf("%s",L_TXT); /* Text String */
             read_txt(uindata,mx.exchange,*place);
@@ -592,6 +664,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
     escape_stdout(rr_hdr.name);
     /* We also show the type if this is an unsupported record type */
     if(rr_hdr.type != RR_A &&
+       rr_hdr.type != RR_AAAA &&
        rr_hdr.type != RR_NS &&
        rr_hdr.type != RR_CNAME &&
        rr_hdr.type != RR_SOA &&
@@ -642,6 +715,23 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
                L_DOT,*(uindata->string + *place + 2),
                L_DOT,*(uindata->string + *place + 3));
         }
+    else if(rr_hdr.type == RR_AAAA) {
+        unsigned short *p;
+        if(uindata->unit_count < *place + 16)
+            return JS_ERROR;
+        /* Display the IP of the data */
+        /* Display the IP of the data */
+        p = (unsigned short*)(uindata->string + *place);
+        printf("|%x:%x:%x:%x:%x:%x:%x:%x\n",
+                   htons(*(p + 0 )),
+                   htons(*(p + 1)),
+                   htons(*(p + 2)),
+                   htons(*(p + 3)),
+                   htons(*(p + 4)),
+                   htons(*(p + 5)),
+                   htons(*(p + 6)),
+                   htons(*(p + 7)));
+        }
     else if(rr_hdr.type == RR_TXT) {
         printf("|");
         read_txt(uindata,mx.exchange,*place);
